defmodule PrymnWeb.Router do
  import Phoenix.LiveDashboard.Router
  import Plug.BasicAuth
  import PrymnWeb.UserAuth

  use PrymnWeb, :router

  pipeline :browser do
    plug :accepts, ["html"]
    plug :fetch_session
    plug :fetch_live_flash
    plug :put_root_layout, html: {PrymnWeb.Layouts, :root}
    plug :protect_from_forgery
    plug :put_secure_browser_headers
    plug :fetch_current_user
  end

  pipeline :api do
    plug :accepts, ["json"]
  end

  pipeline :dev do
    # TODO: Make this safer :)
    plug :basic_auth, username: "hello", password: "secret"
  end

  scope "/", PrymnWeb do
    pipe_through [:browser, :require_authenticated_user]

    get "/", PageController, :home

    live_session :require_authenticated_user,
      on_mount: [{PrymnWeb.UserAuth, :ensure_authenticated}] do
      live "/users/settings", UserSettingsLive
      live "/users/settings/confirm_email/:token", UserSettingsLive

      live "/servers", ServerLive.Index, :index
      live "/servers/new", ServerLive.Index, :new
      live "/servers/:id", ServerLive.Show
    end
  end

  scope "/", PrymnWeb do
    pipe_through [:browser]

    get "/install", PageController, :install
  end

  scope "/auth", PrymnWeb do
    pipe_through [:browser, :redirect_if_user_is_authenticated]

    live_session :redirect_if_user_is_authenticated,
      on_mount: [{PrymnWeb.UserAuth, :redirect_if_user_is_authenticated}],
      layout: {PrymnWeb.Layouts, :auth} do
      live "/register", UserRegistrationLive
      live "/log_in", UserLoginLive
      live "/reset_password", UserForgotPasswordLive
      live "/reset_password/:token", UserResetPasswordLive
    end

    post "/log_in", UserSessionController, :create
  end

  scope "/auth", PrymnWeb do
    pipe_through [:browser]

    live_session :current_user,
      on_mount: [{PrymnWeb.UserAuth, :mount_current_user}],
      layout: {PrymnWeb.Layouts, :auth} do
      live "/confirm", UserConfirmationLive, :new
      live "/confirm/:token", UserConfirmationLive, :edit
    end

    delete "/log_out", UserSessionController, :delete
  end

  scope "/api/v1", PrymnWeb do
    pipe_through :api

    post "/servers/register", ServerController, :register
  end

  scope "/dev" do
    pipe_through [:browser, :dev]

    live_dashboard "/dashboard", metrics: PrymnWeb.Telemetry
    forward "/mailbox", Plug.Swoosh.MailboxPreview
  end
end
