{
  description = "Prymn";

  inputs = {
    nixpkgs.url = "github:NixOS/nixpkgs/nixpkgs-unstable";
    flake-utils.url = "github:numtide/flake-utils";
    rust-overlay = {
      url = "github:oxalica/rust-overlay";
      inputs.nixpkgs.follows = "nixpkgs";
      inputs.flake-utils.follows = "flake-utils";
    };
    crane = {
      url = "github:ipetkov/crane";
      inputs.nixpkgs.follows = "nixpkgs";
      inputs.flake-utils.follows = "flake-utils";
      inputs.rust-overlay.follows = "rust-overlay";
    };
  };

  outputs = { self, nixpkgs, flake-utils, rust-overlay, crane }:
    flake-utils.lib.eachDefaultSystem (system:
      let
        pkgs = import nixpkgs { inherit system overlays; };
        overlays = [ (import ./nix/overlay.nix) (import rust-overlay) ];
        scripts = pkgs.callPackage ./nix/scripts.nix { };
        buildRust = buildTarget: pkgs.callPackage ./nix/rust.nix { inherit crane buildTarget; };
      in
      {
        devShells.default = with pkgs; mkShell {
          packages = [
            elixir
            elixir-ls
            rustToolchain
            protobuf
            protoc-gen-elixir
            scripts.prymn_db
          ] ++ lib.optionals stdenv.isLinux [ inotify-tools ];

          shellHook = ''
            export PROJECT_ROOT_DIR="$PWD"
          '';
        };

        packages.agent.x86_64 = buildRust "x86_64-unknown-linux-musl";
        packages.agent.aarch64 = buildRust "aarch64-unknown-linux-musl";
      });
}
